% tdvExample - minimum working example for computation of tuft deflection angles
%              by means of edge detection (Prewitt method) and line parameterization (Hough transform)
%
% Author: Ben Steinfurth, TU Berlin, Chair of Aerodynamics
% email: ben.steinfurth@tu-berlin.de
% May 2018

clear; close all; clc

%% Declare variables.
rot = atand(0/2240);        % correct (slight) misalignment

roi = [1000,400,300,200];   % region of interest containing only one tuft [xmin,ymin,width,height]

f = 300;                    % sample frequency (acquisition rate of camera)
N = 20;                     % number of photos to evaluate
Phi = zeros(1,N);           % preallocate matrix for tuft deflection angles phi

H = figure;
set(H,'Color','w',...
    'Position',[0 300 1800 400],...
    'PaperSize',[20 5],...
    'PaperPosition',[0 0 20 5],...
    'visible','on')

for j = 1:N
    %% Preprocess images.
    % Load image.
    I  = imread(['example_photos\VN002_',sprintf('%08d',j-1),'.tif']);

    % Rotate image.
    rotI = imrotate(I,rot,'crop');

    % Crop image according to ROI.
    cropRotI = imcrop(rotI,roi);
    
    subplot(1,5,1)
    imshow(cropRotI)
    title('Raw tuft image')

    %% Create a binary image (edge detection).
    BW = edge(cropRotI,'prewitt');
    subplot(1,5,2)
    imshow(BW)
    title('Extracted tuft edges')

    %% Create the Hough transform using the binary image.
    [Ho,Theta,Rho] = hough(BW);
    P  = houghpeaks(Ho,3,'threshold',ceil(0.5*max(Ho(:))));                 % Locate peaks in Hough voting matrix
    subplot(1,5,3)
    imshow(imadjust(mat2gray(Ho)),'XData',Theta,'YData',Rho,...
      'InitialMagnification','fit');
    title('Hough transform of tuft edges')
    xlabel('$\vartheta$ (deg)','interpreter','latex')
    ylabel('$\varrho$ (px)','interpreter','latex')
    colormap bone   
    axis on, axis normal,
    xlim([-90 90]); ylim([-200 350])

    % Parametrize line corresponding to maximum in Hough space.
    lines = houghlines(BW,Theta,Rho,P,'FillGap',10,'MinLength',25);

    %% Find longest segment (used as an approximate for tuft deflection here)
    max_len = 0;
    for k = 1:length(lines)
       xy = [lines(k).point1; lines(k).point2];

       % Determine the longest line segment and its endpoints
       len = norm(lines(k).point1 - lines(k).point2);
       if (len > max_len)
          max_len = len;
          xy_long = xy;
       end
    end
    
    subplot(1,5,4)
    imshow(cropRotI)
    hold on
    plot(xy_long(:,1),xy_long(:,2),'r','linewidth',2);
    title('Extracted line (red)')
    
    % Compute deflection angle based on longest line segment.
    Phi(j) = -acosd((xy_long(2,1)-xy_long(1,1)) / max_len);
    subplot(1,5,5)
    plot(0:(1/f):(j-1)*(1/f),Phi(1:j),'k-o')
    xlim([0 N*(1/f)]); ylim([-35 -10])
    title('Tuft deflection angle')
    xlabel('$t$ (s)','interpreter','latex')
    ylabel('$\varphi$ (deg)','interpreter','latex')
    drawnow

end